﻿local RSwapper3 = LibStub("AceAddon-3.0"):GetAddon("RSwapper3");
local L = LibStub("AceLocale-3.0"):GetLocale("RSwapper3", true);

local RS3GUI = RSwapper3.RS3GUI;

local TabFrame = RS3GUI.TabFrame;
local WeaponFrame = RS3GUI.SubFrame[1];
local OptionsFrame = RS3GUI.SubFrame[2];

function WeaponFrame:DoInit()
	WeaponFrame.DidInit = true;
	
	WeaponFrame:SetFrameStrata("MEDIUM");
	WeaponFrame:SetWidth(RS3GUI.Width);
	WeaponFrame:SetBackdrop({
			bgFile="Interface\\DialogFrame\\UI-DialogBox-Background",
			edgeFile="Interface\\DialogFrame\\UI-DialogBox-Border", 
			tile = true, tileSize = 32, edgeSize = 32, 
			insets = { left = 8, right = 8, top = 8, bottom = 8 }
		});
	WeaponFrame:SetBackdropColor(0,0,0,1);
	WeaponFrame:EnableMouse();
	WeaponFrame:SetPoint("TOPLEFT", TabFrame, "BOTTOMLEFT", 0, 12);
	
	WeaponFrame.WeaponList = {};

	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, L["Dagger"] .. ": ", "Dagger", -13);
	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, L["Dagger"] .. " " .. SECONDARYHANDSLOT .. ": ", "ODagger");
	WeaponFrame:AddWeaponClear(WeaponFrame.WeaponList);
	
	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, L["Sword"] .. ": ", "Sword");
	WeaponFrame:AddWeaponLine(WeaponFrame.WeaponList, L["Sword"] .. " " .. SECONDARYHANDSLOT .. ": ", "OSword");
	WeaponFrame:AddWeaponClear(WeaponFrame.WeaponList);
	
	WeaponFrame.Instructions = WeaponFrame:CreateFontString("$parent_Instructions","OVERLAY","GameFontNormal");
	
	local Anchor = WeaponFrame.WeaponList[#WeaponFrame.WeaponList];
	WeaponFrame.Instructions:SetPoint("TOPLEFT", Anchor.ClearButton or Anchor.Label, "BOTTOMLEFT", 0, -5);
	WeaponFrame.Instructions:SetWidth(RS3GUI.Width - 26);
	WeaponFrame.Instructions:SetText(L["Click the buttons below to set your Dagger weapons. (Stealth)"] .. "\n" .. L["Ctrl-click them to set your Sword weapons. (Non stealth)"] .. "\n" .. L["Right-click them to set your Off Hand weapons. (Optional)"]);
	WeaponFrame.Instructions:SetTextColor(0.2235, 0.6549, 0.8745);
	WeaponFrame.Instructions:SetJustifyH("LEFT");
	
	WeaponFrame.ButtonList = {};

	WeaponFrame:AddWeaponBtn(WeaponFrame.ButtonList, MAINHANDSLOT .. ": ", RSwapper3.nMAINHANDSLOT);
	WeaponFrame:AddWeaponBtn(WeaponFrame.ButtonList, SECONDARYHANDSLOT .. ": ", RSwapper3.nSECONDARYHANDSLOT);
	
	WeaponFrame.BagNum = 0;
	WeaponFrame.BagSlot = 1;
	WeaponFrame.BagDir = 0;
	
	WeaponFrame.BagLabel = WeaponFrame:CreateFontString("$parent_WpnBagLabel", "ARTWORK", "GameFontNormal");
	WeaponFrame.BagLabel:SetText(BAGSLOT .. ": ");
	WeaponFrame.BagLabel:SetPoint("TOPLEFT", WeaponFrame.ButtonList[#WeaponFrame.ButtonList].Label, "BOTTOMLEFT", 0, -5);
	WeaponFrame.BagLabel:SetJustifyV("MIDDLE");
	
	WeaponFrame.BagLeft = CreateFrame("Button", nil, WeaponFrame, "UIPanelButtonTemplate");
	WeaponFrame.BagLeft:SetText("<");
	WeaponFrame.BagLeft:SetHeight(WeaponFrame.BagLeft:GetTextHeight() + 10);
	WeaponFrame.BagLabel:SetHeight(WeaponFrame.BagLeft:GetHeight());
	WeaponFrame.BagLeft:SetWidth(WeaponFrame.BagLeft:GetTextWidth() + 10);
	WeaponFrame.BagLeft:SetPoint("TOPLEFT", WeaponFrame.BagLabel, "TOPRIGHT");
	WeaponFrame.BagLeft:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WeaponFrame, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", WeaponFrame.BagLeft, "BOTTOMLEFT");
			GameTooltip:ClearLines();
			GameTooltip:AddLine(PREVIOUS);
			GameTooltip:Show();
		end);
	WeaponFrame.BagLeft:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
	WeaponFrame.BagLeft:SetScript("OnClick", function()
			WeaponFrame.BagDir = -1;
			WeaponFrame:DoUpdate();
		end);
			
	WeaponFrame.BagRight = CreateFrame("Button", nil, WeaponFrame, "UIPanelButtonTemplate");
	WeaponFrame.BagRight:SetText(">");
	WeaponFrame.BagRight:SetPoint("TOPLEFT", WeaponFrame.BagLeft, "TOPRIGHT");
	WeaponFrame.BagRight:SetHeight(WeaponFrame.BagRight:GetTextHeight() + 10);
	WeaponFrame.BagRight:SetWidth(WeaponFrame.BagRight:GetTextWidth() + 10);
	WeaponFrame.BagRight:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WeaponFrame, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", WeaponFrame.BagRight, "BOTTOMLEFT");
			GameTooltip:ClearLines();
			GameTooltip:AddLine(NEXT);
			GameTooltip:Show();
		end);
	WeaponFrame.BagRight:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
	WeaponFrame.BagRight:SetScript("OnClick", function()
			WeaponFrame.BagDir = 1;
			WeaponFrame:DoUpdate();
		end);
	
	WeaponFrame.BagBtn = CreateFrame("Button", nil, WeaponFrame, "UIPanelButtonTemplate");
	WeaponFrame.BagBtn:SetPoint("TOPLEFT", WeaponFrame.BagRight, "TOPRIGHT", 5, 0);
	WeaponFrame.BagBtn:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	WeaponFrame.BagBtn:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WeaponFrame, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", WeaponFrame, "TOPRIGHT");
			GameTooltip:ClearLines();
			if (WeaponFrame.BagLink) then
				GameTooltip:SetHyperlink(WeaponFrame.BagLink);
			else
				GameTooltip:AddLine(EMPTY);
			end
			GameTooltip:Show();
		end);
	WeaponFrame.BagBtn:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
	WeaponFrame.BagBtn:SetScript("OnClick", function(self, button)
			if (WeaponFrame.BagLink) then
				local sword = IsControlKeyDown();
				if (button == "LeftButton") then
					if (sword) then
						RSwapper3.db.char.Sword = WeaponFrame.BagLink;
					else
						RSwapper3.db.char.Dagger = WeaponFrame.BagLink;
					end
				elseif (button == "RightButton") then
					if (sword) then
						RSwapper3.db.char.OSword = WeaponFrame.BagLink;
					else
						RSwapper3.db.char.ODagger = WeaponFrame.BagLink;
					end
				end
				WeaponFrame:DoUpdate();
			end
		end);
end

function WeaponFrame:AddWeaponLine(list, label, value, ypos)
	local index = (#list) + 1;
	
	local WpnLine = {};
	WpnLine.Value = value;
	
	WpnLine.Label = WeaponFrame:CreateFontString("$parent_WpnLabel" .. index, "ARTWORK", "GameFontNormal");
	WpnLine.Text = WeaponFrame:CreateFontString("$parent_WpnText" .. index, "ARTWORK", "GameFontNormal");
	WpnLine.Tip = CreateFrame("Frame", nil, WeaponFrame);
	
	if (ypos) then
		WpnLine.Label:SetPoint("TOPLEFT", WeaponFrame, "TOPLEFT", 13, ypos);
	else
		if (list[index-1].ClearButton) then
			WpnLine.Label:SetPoint("TOPLEFT", list[index-1].ClearButton, "BOTTOMLEFT", 0, -5);
		else
			WpnLine.Label:SetPoint("TOPLEFT", list[index-1].Label, "BOTTOMLEFT", 0, -5);
		end
	end
	WpnLine.Label:SetText(label);
	WpnLine.Label:SetJustifyH("LEFT");
	WpnLine.Label:SetJustifyV("MIDDLE");
		
	WpnLine.Text:SetPoint("TOPLEFT", WpnLine.Label, "TOPRIGHT", 0, 0);
	WpnLine.Text:SetWidth(RS3GUI.Width - 26 - WpnLine.Label:GetWidth());
	WpnLine.Text:SetJustifyH("LEFT");
	
	WpnLine.Tip:SetPoint("TOPLEFT", WpnLine.Text);
	WpnLine.Tip:EnableMouse();
	
	WpnLine.Tip:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WeaponFrame, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", WeaponFrame, "TOPRIGHT");
			GameTooltip:ClearLines();
			if (RSwapper3.db.char[value]) then
				GameTooltip:SetHyperlink(RSwapper3.db.char[value]);
			else
				GameTooltip:AddLine(L["|cffff0000[Not Set]|r"]);
			end
			GameTooltip:Show();
		end);
	WpnLine.Tip:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
		
	list[index] = WpnLine;
end

function WeaponFrame:AddWeaponClear(list)
	local index = (#list);
	
	local ClrBtn = CreateFrame("Button", nil, WeaponFrame, "UIPanelButtonTemplate");
	ClrBtn:SetPoint("TOPLEFT", list[index].Label, "BOTTOMLEFT", 0, -5);
	ClrBtn:SetText(REMOVE);
	ClrBtn:SetWidth(ClrBtn:GetTextWidth() + 20);
	ClrBtn:SetHeight(ClrBtn:GetTextHeight() + 10);
	
	ClrBtn:SetScript("OnClick", function(self, button)
			RSwapper3.db.char[list[index].Value] = nil;
			WeaponFrame:DoUpdate();
		end);
	
	list[index].ClearButton = ClrBtn;
end

function WeaponFrame:AddWeaponBtn(list, label, slot)
	local index = (#list) + 1;
	
	local BtnLine = {};
	BtnLine.Label = WeaponFrame:CreateFontString("$parent_WpnBtnLabel" .. index, "ARTWORK", "GameFontNormal");
	if (index == 1) then
		BtnLine.Label:SetPoint("TOPLEFT", WeaponFrame.Instructions, "BOTTOMLEFT", 0, -5);
	else
		BtnLine.Label:SetPoint("TOPLEFT", list[index-1].Label, "BOTTOMLEFT", 0, -5);
	end
	
	BtnLine.Slot = slot;
	
	BtnLine.Label:SetText(label);
	BtnLine.Label:SetJustifyV("MIDDLE");
	
	BtnLine.Button = CreateFrame("Button", nil, WeaponFrame, "UIPanelButtonTemplate");
	BtnLine.Button:SetPoint("TOPLEFT", BtnLine.Label, "TOPRIGHT");
	BtnLine.Button:SetText(L["|cffff0000[Not Set]|r"]);
	BtnLine.Button:SetWidth(BtnLine.Button:GetTextWidth() + 20);
	BtnLine.Button:SetHeight(BtnLine.Button:GetTextHeight() + 10);
	BtnLine.Button:RegisterForClicks("LeftButtonUp", "RightButtonUp");
	
	BtnLine.Button:SetScript("OnEnter", function()
			GameTooltip:SetOwner(WeaponFrame, "ANCHOR_NONE");
			GameTooltip:SetPoint("TOPLEFT", WeaponFrame, "TOPRIGHT");
			GameTooltip:ClearLines();
			if (BtnLine.Link) then
				GameTooltip:SetHyperlink(BtnLine.Link);
			else
				GameTooltip:AddLine(EMPTY);
			end
			GameTooltip:Show();
		end);
	BtnLine.Button:SetScript("OnLeave", function()
			GameTooltip:Hide();
		end);
	BtnLine.Button:SetScript("OnClick", function(self, button)
			if (BtnLine.Link) then
				local sword = IsControlKeyDown();
				if (button == "LeftButton") then
					if (sword) then
						RSwapper3.db.char.Sword = BtnLine.Link;
					else
						RSwapper3.db.char.Dagger = BtnLine.Link;
					end
				elseif (button == "RightButton") then
					if (sword) then
						RSwapper3.db.char.OSword = BtnLine.Link;
					else
						RSwapper3.db.char.ODagger = BtnLine.Link;
					end
				end
				WeaponFrame:DoUpdate();
			end
		end);
	
	list[index] = BtnLine;
end

function WeaponFrame:DoUpdate()
	local frameheight, labelheight, textwidth, textheight, maxwidth = 13, 0, 0, 0, 0;
	for i, WpnLine in ipairs(WeaponFrame.WeaponList) do
		labelheight = WpnLine.Label:GetStringHeight();
		
		WpnLine.Text:SetText(RSwapper3.db.char[WpnLine.Value] or L["|cffff0000[Not Set]|r"]);
		textheight = WpnLine.Text:GetHeight();
		if (labelheight > textheight) then
			textheight = labelheight;
		end
		WpnLine.Label:SetHeight(textheight);
		
		maxwidth = WpnLine.Text:GetWidth();
		textwidth = WpnLine.Text:GetStringWidth();
		if (maxwidth < textwidth) then
			textwidth = maxwidth;
		end
		
		WpnLine.Tip:SetWidth(textwidth);
		WpnLine.Tip:SetHeight(textheight);
		
		frameheight = frameheight + textheight + 5;
		if (WpnLine.ClearButton) then
			frameheight = frameheight + WpnLine.ClearButton:GetHeight() + 5;
		end
	end
	
	frameheight = frameheight + WeaponFrame.Instructions:GetHeight() + 5;
	
	for i, BtnLine in ipairs(WeaponFrame.ButtonList) do
		BtnLine.Link = GetInventoryItemLink("player", BtnLine.Slot);
		BtnLine.Button:SetText(BtnLine.Link or EMPTY);
		BtnLine.Button:SetWidth(BtnLine.Button:GetTextWidth() + 20);
		BtnLine.Button:SetHeight(BtnLine.Button:GetTextHeight() + 10);
		BtnLine.Label:SetHeight(BtnLine.Button:GetHeight());
		
		frameheight = frameheight + BtnLine.Button:GetHeight() + 5;
	end
	
	local found, passed, link, itemtype, itemsubtype, _;
	local firstwpn = {};
	local prevwpn = {};
	WeaponFrame.BagLink = nil;
	for bag = 0, 4 do
		local slots = GetContainerNumSlots(bag);
		for slot = 1, slots do
			link = GetContainerItemLink(bag, slot);
			if (link) then
				_, _, _, _, _, itemtype, itemsubtype = GetItemInfo(link);
				if ((itemtype == RSwapper3.LWEAPON) and (itemsubtype ~= RSwapper3.LFISHINGPOLES)) then
					if (not firstwpn.bag) then
						firstwpn.bag = bag;
						firstwpn.slot = slot;
					end
					--If we've reached the last used item, or passed it
					if ((not passed) and ((bag > WeaponFrame.BagNum) or ((bag == WeaponFrame.BagNum) and (slot >= WeaponFrame.BagSlot)))) then
						passed = true;
						if (WeaponFrame.BagDir == 0) then
							WeaponFrame.BagLink = link;
							WeaponFrame.BagNum = bag;
							WeaponFrame.BagSlot = slot;
							found = true;
							break;
						elseif (WeaponFrame.BagDir < 0) then
							if (prevwpn.bag) then
								WeaponFrame.BagLink = GetContainerItemLink(prevwpn.bag, prevwpn.slot);
								WeaponFrame.BagNum = prevwpn.bag;
								WeaponFrame.BagSlot = prevwpn.slot;
								found = true;
								break;
							end
						end
					end
					--If we've passed the last used item
					if ((bag > WeaponFrame.BagNum) or ((bag == WeaponFrame.BagNum) and (slot > WeaponFrame.BagSlot))) then
						if (WeaponFrame.BagDir > 0) then
							WeaponFrame.BagLink = link;
							WeaponFrame.BagNum = bag;
							WeaponFrame.BagSlot = slot;
							found = true;
							break;
						end
					end
					prevwpn.bag = bag;
					prevwpn.slot = slot;
				end
			end
		end
		if (found) then
			break;
		end
	end
	
	if (not WeaponFrame.BagLink) then
		if ((WeaponFrame.BagDir < 0) and (prevwpn.bag)) then
			WeaponFrame.BagLink = GetContainerItemLink(prevwpn.bag, prevwpn.slot);
			WeaponFrame.BagNum = prevwpn.bag;
			WeaponFrame.BagSlot = prevwpn.slot;
		elseif (firstwpn.bag) then
			WeaponFrame.BagLink = GetContainerItemLink(firstwpn.bag, firstwpn.slot);
			WeaponFrame.BagNum = firstwpn.bag;
			WeaponFrame.BagSlot = firstwpn.slot;
		end
	end
	
	WeaponFrame.BagDir = 0;
	
	if (WeaponFrame.BagLink) then
		WeaponFrame.BagBtn:SetText(WeaponFrame.BagLink);
		WeaponFrame.BagBtn:SetHeight(WeaponFrame.BagBtn:GetTextHeight() + 10);
		WeaponFrame.BagBtn:SetWidth(WeaponFrame.BagBtn:GetTextWidth() + 20);
		WeaponFrame.BagLeft:SetHeight(WeaponFrame.BagBtn:GetHeight());
		WeaponFrame.BagRight:SetHeight(WeaponFrame.BagBtn:GetHeight());
	else
		WeaponFrame.BagBtn:SetText(EMPTY);
	end
	
	WeaponFrame:SetHeight(frameheight + WeaponFrame.BagBtn:GetHeight() + 13);
end